@@ -15,7 +15,6 @@
 /// \brief C++ Tracing in VCD Format
 ///
 //=============================================================================
-// SPDIFF_OFF
 
 // clang-format off
 
@@ -34,8 +33,6 @@
 # include <unistd.h>
 #endif
 
-// SPDIFF_ON
-
 #ifndef O_LARGEFILE  // For example on WIN32
 # define O_LARGEFILE 0
 #endif
@@ -100,14 +97,14 @@ VerilatedVcd::VerilatedVcd(VerilatedVcdFile* filep) {
     m_suffixesp = nullptr;
 }
 
-void VerilatedVcd::open(const char* filename) {
-    m_assertOne.check();
+void VerilatedVcd::open(const char* filename) VL_MT_SAFE_EXCLUDES(m_mutex) {
+    const VerilatedLockGuard lock(m_mutex);
     if (isOpen()) return;
 
     // Set member variables
     m_filename = filename;  // "" is ok, as someone may overload open
 
-    openNext(m_rolloverMB != 0);
+    openNextImp(m_rolloverMB != 0);
     if (!isOpen()) return;
 
     dumpHeader();
@@ -116,13 +113,17 @@ void VerilatedVcd::open(const char* filename) {
     m_suffixesp = &m_suffixes[0];  // Note: C++11 m_suffixes.data();
 
     // When using rollover, the first chunk contains the header only.
-    if (m_rolloverMB) openNext(true);
+    if (m_rolloverMB) openNextImp(true);
 }
 
-void VerilatedVcd::openNext(bool incFilename) {
+void VerilatedVcd::openNext(bool incFilename) VL_MT_SAFE_EXCLUDES(m_mutex) {
     // Open next filename in concat sequence, mangle filename if
     // incFilename is true.
-    m_assertOne.check();
+    const VerilatedLockGuard lock(m_mutex);
+    openNextImp(incFilename);
+}
+
+void VerilatedVcd::openNextImp(bool incFilename) {
     closePrev();  // Close existing
     if (incFilename) {
         // Find _0000.{ext} in filename
@@ -138,7 +139,9 @@ void VerilatedVcd::openNext(bool incFilename) {
                     name[pos - 2] = '0';
                     if ((++(name[pos - 3])) > '9') {
                         name[pos - 3] = '0';
-                        if ((++(name[pos - 4])) > '9') { name[pos - 4] = '0'; }
+                        if ((++(name[pos - 4])) > '9') {  //
+                            name[pos - 4] = '0';
+                        }
                     }
                 }
             }
@@ -164,7 +167,7 @@ void VerilatedVcd::openNext(bool incFilename) {
 }
 
 bool VerilatedVcd::preChangeDump() {
-    if (VL_UNLIKELY(m_rolloverMB && m_wroteBytes > m_rolloverMB)) { openNext(true); }
+    if (VL_UNLIKELY(m_rolloverMB && m_wroteBytes > m_rolloverMB)) openNextImp(true);
     return isOpen();
 }
 
@@ -220,7 +223,7 @@ void VerilatedVcd::closePrev() {
     // This function is on the flush() call path
     if (!isOpen()) return;
 
-    VerilatedTrace<VerilatedVcd>::flush();
+    VerilatedTrace<VerilatedVcd>::flushBase();
     bufferFlush();
     m_isOpen = false;
     m_filep->close();
@@ -237,9 +240,9 @@ void VerilatedVcd::closeErr() {
     m_filep->close();  // May get error, just ignore it
 }
 
-void VerilatedVcd::close() {
+void VerilatedVcd::close() VL_MT_SAFE_EXCLUDES(m_mutex) {
     // This function is on the flush() call path
-    m_assertOne.check();
+    const VerilatedLockGuard lock(m_mutex);
     if (!isOpen()) return;
     if (m_evcd) {
         printStr("$vcdclose ");
@@ -249,11 +252,12 @@ void VerilatedVcd::close() {
     closePrev();
     // closePrev() called VerilatedTrace<VerilatedVcd>::flush(), so we just
     // need to shut down the tracing thread here.
-    VerilatedTrace<VerilatedVcd>::close();
+    VerilatedTrace<VerilatedVcd>::closeBase();
 }
 
-void VerilatedVcd::flush() {
-    VerilatedTrace<VerilatedVcd>::flush();
+void VerilatedVcd::flush() VL_MT_SAFE_EXCLUDES(m_mutex) {
+    const VerilatedLockGuard lock(m_mutex);
+    VerilatedTrace<VerilatedVcd>::flushBase();
     bufferFlush();
 }
 
@@ -267,7 +271,7 @@ void VerilatedVcd::printStr(const char* str) {
 
 void VerilatedVcd::printQuad(vluint64_t n) {
     char buf[100];
-    sprintf(buf, "%" VL_PRI64 "u", n);
+    VL_SNPRINTF(buf, 100, "%" VL_PRI64 "u", n);
     printStr(buf);
 }
 
@@ -291,7 +295,6 @@ void VerilatedVcd::bufferFlush() VL_MT_UNSAFE_ONE {
     // We add output data to m_writep.
     // When it gets nearly full we dump it using this routine which calls write()
     // This is much faster than using buffered I/O
-    m_assertOne.check();
     if (VL_UNLIKELY(!isOpen())) return;
     char* wp = m_wrBufp;
     while (true) {
@@ -345,9 +348,16 @@ void VerilatedVcd::printIndent(int level_change) {
 
 void VerilatedVcd::dumpHeader() {
     printStr("$version Generated by VerilatedVcd $end\n");
-    time_t time_str = time(nullptr);
     printStr("$date ");
-    printStr(ctime(&time_str));
+    {
+        time_t tick = time(nullptr);
+        tm ticktm;
+        VL_LOCALTIME_R(&tick, &ticktm);
+        constexpr int bufsize = 50;
+        char buf[bufsize];
+        strftime(buf, bufsize, "%c", &ticktm);
+        printStr(buf);
+    }
     printStr(" $end\n");
 
     printStr("$timescale ");
@@ -401,13 +411,22 @@ void VerilatedVcd::dumpHeader() {
             if (*np == ' ') np++;
             if (*np == '\t') break;  // tab means signal name starts
             printIndent(1);
-            printStr("$scope module ");
+            // Find character after name end
+            const char* sp = np;
+            while (*sp && *sp != ' ' && *sp != '\t' && *sp != '\f') sp++;
+
+            if (*sp == '\f') {
+                printStr("$scope struct ");
+            } else {
+                printStr("$scope module ");
+            }
+
             for (; *np && *np != ' ' && *np != '\t'; np++) {
                 if (*np == '[') {
                     printStr("(");
                 } else if (*np == ']') {
                     printStr(")");
-                } else {
+                } else if (*np != '\f') {
                     *m_writep++ = *np;
                 }
             }
@@ -476,11 +495,12 @@ void VerilatedVcd::declare(vluint32_t code, const char* name, const char* wirep,
         decl += wirep;  // usually "wire"
     }
 
-    char buf[1000];
-    sprintf(buf, " %2d ", bits);
+    constexpr size_t bufsize = 1000;
+    char buf[bufsize];
+    VL_SNPRINTF(buf, bufsize, " %2d ", bits);
     decl += buf;
     if (m_evcd) {
-        sprintf(buf, "<%u", code);
+        VL_SNPRINTF(buf, bufsize, "<%u", code);
         decl += buf;
     } else {
         // Add string code to decl
@@ -494,7 +514,8 @@ void VerilatedVcd::declare(vluint32_t code, const char* name, const char* wirep,
         // 1 bit values don't have a ' ' separator between value and string code
         const bool isBit = bits == 1;
         entryp[0] = ' ';  // Separator
-        std::strcpy(entryp + !isBit, buf);  // Code (overwrite separator if isBit)
+        // Use memcpy as we checked size above, and strcpy is flagged unsafe
+        std::memcpy(entryp + !isBit, buf, strlen(buf));  // Code (overwrite separator if isBit)
         entryp[length + !isBit] = '\n';  // Replace '\0' with line termination '\n'
         // Set length of suffix (used to increment write pointer)
         entryp[VL_TRACE_SUFFIX_ENTRY_SIZE - 1] = !isBit + length + 1;
@@ -502,12 +523,12 @@ void VerilatedVcd::declare(vluint32_t code, const char* name, const char* wirep,
     decl += " ";
     decl += basename;
     if (array) {
-        sprintf(buf, "(%d)", arraynum);
+        VL_SNPRINTF(buf, bufsize, "(%d)", arraynum);
         decl += buf;
         hiername += buf;
     }
     if (bussed) {
-        sprintf(buf, " [%d:%d]", msb, lsb);
+        VL_SNPRINTF(buf, bufsize, " [%d:%d]", msb, lsb);
         decl += buf;
     }
     decl += " $end\n";
@@ -645,8 +666,8 @@ void VerilatedVcd::emitWData(vluint32_t code, const WData* newvalp, int bits) {
 VL_ATTR_ALWINLINE
 void VerilatedVcd::emitDouble(vluint32_t code, double newval) {
     char* wp = m_writep;
-    // Buffer can't overflow before sprintf; we sized during declaration
-    sprintf(wp, "r%.16g", newval);
+    // Buffer can't overflow before VL_SNPRINTF; we sized during declaration
+    VL_SNPRINTF(wp, m_wrChunkSize, "r%.16g", newval);
     wp += strlen(wp);
     finishLine(code, wp);
 }
@@ -759,8 +780,8 @@ void VerilatedVcd::fullTriArray(vluint32_t code, const vluint32_t* newvalp,
 void VerilatedVcd::fullDouble(vluint32_t code, const double newval) {
     // cppcheck-suppress invalidPointerCast
     (*(reinterpret_cast<double*>(oldp(code)))) = newval;
-    // Buffer can't overflow before sprintf; we sized during declaration
-    sprintf(m_writep, "r%.16g", newval);
+    // Buffer can't overflow before VL_SNPRINTF; we sized during declaration
+    VL_SNPRINTF(m_writep, m_wrChunkSize, "r%.16g", newval);
     m_writep += strlen(m_writep);
     *m_writep++ = ' ';
     m_writep = writeCode(m_writep, code);
@@ -858,6 +879,10 @@ void vcdTestMain(const char* filenamep) {
     {
         VerilatedVcdC* vcdp = new VerilatedVcdC;
         vcdp->evcd(true);
+        vcdp->set_time_unit("1ms");
+        vcdp->set_time_unit(std::string("1ms"));
+        vcdp->set_time_resolution("1ns");
+        vcdp->set_time_resolution(std::string("1ns"));
         vcdp->spTrace()->addInitCb(&vcdInit, 0);
         vcdp->spTrace()->addFullCb(&vcdFull, 0);
         vcdp->spTrace()->addChgCb(&vcdChange, 0);
